@@ -6,7 +6,7 @@
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -17,6 +17,7 @@
 package org.apache.commons.lang3;
 
 import static org.apache.commons.lang3.JavaVersion.JAVA_1_4;
+import static org.apache.commons.lang3.LangAssertions.assertIllegalArgumentException;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
@@ -30,6 +31,7 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.Modifier;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Comparator;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Locale;
@@ -39,11 +41,12 @@ import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.MethodSource;
+import org.junitpioneer.jupiter.DefaultLocale;
 
 /**
- * Unit tests for {@link LocaleUtils}.
+ * Tests for {@link LocaleUtils}.
  */
-public class LocaleUtilsTest extends AbstractLangTest {
+class LocaleUtilsTest extends AbstractLangTest {
 
     private static final Locale LOCALE_EN = new Locale("en", "");
     private static final Locale LOCALE_EN_US = new Locale("en", "US");
@@ -72,7 +75,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
         for (final String country : countries) {
             boolean found = false;
             // see if it was returned by the set
-            for (Locale locale : list) {
+            for (final Locale locale : list) {
                 // should have an en empty variant
                 assertTrue(StringUtils.isEmpty(locale.getVariant()));
                 assertEquals(language, locale.getLanguage());
@@ -104,7 +107,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
         for (final String language : languages) {
             boolean found = false;
             // see if it was returned by the set
-            for (Locale locale : list) {
+            for (final Locale locale : list) {
                 // should have an en empty variant
                 assertTrue(StringUtils.isEmpty(locale.getVariant()));
                 assertEquals(country, locale.getCountry());
@@ -200,7 +203,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test availableLocaleList() method.
      */
     @Test
-    public void testAvailableLocaleList() {
+    void testAvailableLocaleList() {
         final List<Locale> list = LocaleUtils.availableLocaleList();
         final List<Locale> list2 = LocaleUtils.availableLocaleList();
         assertNotNull(list);
@@ -208,7 +211,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
         assertUnmodifiableCollection(list);
 
         final Locale[] jdkLocaleArray = Locale.getAvailableLocales();
-        final List<Locale> jdkLocaleList = Arrays.asList(jdkLocaleArray);
+        final List<Locale> jdkLocaleList = Arrays.asList(ArraySorter.sort(jdkLocaleArray, Comparator.comparing(Locale::toString)));
         assertEquals(jdkLocaleList, list);
     }
 
@@ -216,7 +219,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test availableLocaleSet() method.
      */
     @Test
-    public void testAvailableLocaleSet() {
+    void testAvailableLocaleSet() {
         final Set<Locale> set = LocaleUtils.availableLocaleSet();
         final Set<Locale> set2 = LocaleUtils.availableLocaleSet();
         assertNotNull(set);
@@ -233,7 +236,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test that constructors are public, and work, etc.
      */
     @Test
-    public void testConstructor() {
+    void testConstructor() {
         assertNotNull(new LocaleUtils());
         final Constructor<?>[] cons = LocaleUtils.class.getDeclaredConstructors();
         assertEquals(1, cons.length);
@@ -246,7 +249,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test countriesByLanguage() method.
      */
     @Test
-    public void testCountriesByLanguage() {
+    void testCountriesByLanguage() {
         assertCountriesByLanguage(null, new String[0]);
         assertCountriesByLanguage("de", new String[]{"DE", "CH", "AT", "LU"});
         assertCountriesByLanguage("zz", new String[0]);
@@ -258,7 +261,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      */
     @SuppressWarnings("boxing") // JUnit4 does not support primitive equality testing apart from long
     @Test
-    public void testIsAvailableLocale() {
+    void testIsAvailableLocale() {
         final Set<Locale> set = LocaleUtils.availableLocaleSet();
         assertEquals(set.contains(LOCALE_EN), LocaleUtils.isAvailableLocale(LOCALE_EN));
         assertEquals(set.contains(LOCALE_EN_US), LocaleUtils.isAvailableLocale(LOCALE_EN_US));
@@ -270,7 +273,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
     }
 
     @Test
-    public void testIsLanguageUndetermined() {
+    void testIsLanguageUndetermined() {
         final Set<Locale> set = LocaleUtils.availableLocaleSet();
         // Determined
         assertNotEquals(set.contains(LOCALE_EN), LocaleUtils.isLanguageUndetermined(LOCALE_EN));
@@ -289,7 +292,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Tests #LANG-328 - only language+variant
      */
     @Test
-    public void testLang328() {
+    void testLang328() {
         assertValidToLocale("fr__P", "fr", "", "P");
         assertValidToLocale("fr__POSIX", "fr", "", "POSIX");
     }
@@ -298,42 +301,22 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Tests #LANG-865, strings starting with an underscore.
      */
     @Test
-    public void testLang865() {
+    void testLang865() {
         assertValidToLocale("_GB", "", "GB", "");
         assertValidToLocale("_GB_P", "", "GB", "P");
         assertValidToLocale("_GB_POSIX", "", "GB", "POSIX");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("_G"),
-                "Must be at least 3 chars if starts with underscore");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("_Gb"),
-                "Must be uppercase if starts with underscore");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("_gB"),
-                "Must be uppercase if starts with underscore");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("_1B"),
-                "Must be letter if starts with underscore");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("_G1"),
-                "Must be letter if starts with underscore");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("_GB_"),
-                "Must be at least 5 chars if starts with underscore");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("_GBAP"),
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("_G"), "Must be at least 3 chars if starts with underscore");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("_Gb"), "Must be uppercase if starts with underscore");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("_gB"), "Must be uppercase if starts with underscore");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("_1B"), "Must be letter if starts with underscore");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("_G1"), "Must be letter if starts with underscore");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("_GB_"), "Must be at least 5 chars if starts with underscore");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("_GBAP"),
                 "Must have underscore after the country if starts with underscore and is at least 5 chars");
     }
 
     @Test
-    public void testLanguageAndUNM49Numeric3AreaCodeLang1312() {
+    void testLanguageAndUNM49Numeric3AreaCodeLang1312() {
         assertValidToLocale("en_001", "en", "001");
         assertValidToLocale("en_150", "en", "150");
         assertValidToLocale("ar_001", "ar", "001");
@@ -347,7 +330,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test languagesByCountry() method.
      */
     @Test
-    public void testLanguagesByCountry() {
+    void testLanguagesByCountry() {
         assertLanguageByCountry(null, new String[0]);
         assertLanguageByCountry("GB", new String[]{"en"});
         assertLanguageByCountry("ZZ", new String[0]);
@@ -358,7 +341,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test localeLookupList() method.
      */
     @Test
-    public void testLocaleLookupList_Locale() {
+    void testLocaleLookupList_Locale() {
         assertLocaleLookupList(null, null, new Locale[0]);
         assertLocaleLookupList(LOCALE_QQ, null, new Locale[]{LOCALE_QQ});
         assertLocaleLookupList(LOCALE_EN, null, new Locale[]{LOCALE_EN});
@@ -378,7 +361,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test localeLookupList() method.
      */
     @Test
-    public void testLocaleLookupList_LocaleLocale() {
+    void testLocaleLookupList_LocaleLocale() {
         assertLocaleLookupList(LOCALE_QQ, LOCALE_QQ,
                 new Locale[]{LOCALE_QQ});
         assertLocaleLookupList(LOCALE_EN, LOCALE_EN,
@@ -430,7 +413,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
 
     @ParameterizedTest
     @MethodSource("java.util.Locale#getAvailableLocales")
-    public void testParseAllLocales(final Locale actualLocale) {
+    void testParseAllLocales(final Locale actualLocale) {
         // Check if it's possible to recreate the Locale using just the standard constructor
         final Locale locale = new Locale(actualLocale.getLanguage(), actualLocale.getCountry(), actualLocale.getVariant());
         if (actualLocale.equals(locale)) { // it is possible for LocaleUtils.toLocale to handle these Locales
@@ -442,7 +425,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
             }
             String localeStr = str;
             if (suff >= 0) { // we have a suffix
-                assertThrows(IllegalArgumentException.class, () -> LocaleUtils.toLocale(str));
+                assertIllegalArgumentException(() -> LocaleUtils.toLocale(str));
                 // try without suffix
                 localeStr = str.substring(0, suff);
             }
@@ -455,7 +438,7 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test for 3-chars locale, further details at LANG-915
      */
     @Test
-    public void testThreeCharsLocale() {
+    void testThreeCharsLocale() {
         for (final String str : Arrays.asList("udm", "tet")) {
             final Locale locale = LocaleUtils.toLocale(str);
             assertNotNull(locale);
@@ -469,9 +452,8 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test toLocale(String) method.
      */
     @Test
-    public void testToLocale_1Part() {
+    void testToLocale_1Part() {
         assertNull(LocaleUtils.toLocale((String) null));
-
         assertValidToLocale("us");
         assertValidToLocale("fr");
         assertValidToLocale("de");
@@ -480,53 +462,34 @@ public class LocaleUtilsTest extends AbstractLangTest {
         assertValidToLocale("qq");
         // LANG-941: JDK 8 introduced the empty locale as one of the default locales
         assertValidToLocale("");
-
-        assertThrows(IllegalArgumentException.class, () -> LocaleUtils.toLocale("Us"), "Should fail if not lowercase");
-        assertThrows(IllegalArgumentException.class, () -> LocaleUtils.toLocale("uS"), "Should fail if not lowercase");
-        assertThrows(IllegalArgumentException.class, () -> LocaleUtils.toLocale("u#"), "Should fail if not lowercase");
-        assertThrows(
-                IllegalArgumentException.class, () -> LocaleUtils.toLocale("u"), "Must be 2 chars if less than 5");
-        assertThrows(
-                IllegalArgumentException.class, () -> LocaleUtils.toLocale("uu_U"), "Must be 2 chars if less than 5");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("Us"), "Should fail if not lowercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("uS"), "Should fail if not lowercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("u#"), "Should fail if not lowercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("u"), "Must be 2 chars if less than 5");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("uu_U"), "Must be 2 chars if less than 5");
     }
 
     /**
      * Test toLocale() method.
      */
     @Test
-    public void testToLocale_2Part() {
+    void testToLocale_2Part() {
         assertValidToLocale("us_EN", "us", "EN");
         assertValidToLocale("us-EN", "us", "EN");
-        //valid though doesn't exist
+        // valid though doesn't exist
         assertValidToLocale("us_ZH", "us", "ZH");
-
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("us_En"),
-                "Should fail second part not uppercase");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("us_en"),
-                "Should fail second part not uppercase");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("us_eN"),
-                "Should fail second part not uppercase");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("uS_EN"),
-                "Should fail first part not lowercase");
-        assertThrows(
-                IllegalArgumentException.class,
-                () -> LocaleUtils.toLocale("us_E3"),
-                "Should fail second part not uppercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("us_En"), "Should fail second part not uppercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("us_en"), "Should fail second part not uppercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("us_eN"), "Should fail second part not uppercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("uS_EN"), "Should fail first part not lowercase");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("us_E3"), "Should fail second part not uppercase");
     }
 
     /**
      * Test toLocale() method.
      */
     @Test
-    public void testToLocale_3Part() {
+    void testToLocale_3Part() {
         assertValidToLocale("us_EN_A", "us", "EN", "A");
         assertValidToLocale("us-EN-A", "us", "EN", "A");
         // this isn't pretty, but was caused by a jdk bug it seems
@@ -538,8 +501,8 @@ public class LocaleUtilsTest extends AbstractLangTest {
             assertValidToLocale("us_EN_a", "us", "EN", "A");
             assertValidToLocale("us_EN_SFsafdFDsdfF", "us", "EN", "SFSAFDFDSDFF");
         }
-        assertThrows(IllegalArgumentException.class, () -> LocaleUtils.toLocale("us_EN-a"), "Should fail as no consistent delimiter");
-        assertThrows(IllegalArgumentException.class, () -> LocaleUtils.toLocale("uu_UU_"), "Must be 3, 5 or 7+ in length");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("us_EN-a"), "Should fail as no consistent delimiter");
+        assertIllegalArgumentException(() -> LocaleUtils.toLocale("uu_UU_"), "Must be 3, 5 or 7+ in length");
         // LANG-1741
         assertEquals(new Locale("en", "001", "US_POSIX"), LocaleUtils.toLocale("en_001_US_POSIX"));
     }
@@ -548,18 +511,49 @@ public class LocaleUtilsTest extends AbstractLangTest {
      * Test toLocale(Locale) method.
      */
     @Test
-    public void testToLocale_Locale_defaults() {
+    void testToLocale_Locale_defaults() {
         assertNull(LocaleUtils.toLocale((String) null));
         assertEquals(Locale.getDefault(), LocaleUtils.toLocale((Locale) null));
         assertEquals(Locale.getDefault(), LocaleUtils.toLocale(Locale.getDefault()));
     }
 
+    @ParameterizedTest
+    @MethodSource("java.util.Locale#getISOCountries")
+    void testToLocaleGetIso3Country(final String country) {
+        assertEquals(LocaleUtils.ofCountry(country).getISO3Country(), LocaleUtils.toLocale(country).getISO3Country());
+    }
+
+    @Test
+    void testToLocaleGetIso3CountryKnown() {
+        assertEquals("USA", LocaleUtils.toLocale("US").getISO3Country());
+        assertEquals("GBR", LocaleUtils.toLocale("GB").getISO3Country());
+        assertEquals("PAK", LocaleUtils.toLocale("PK").getISO3Country());
+        assertEquals("IND", LocaleUtils.toLocale("IN").getISO3Country());
+        assertEquals("FRA", LocaleUtils.toLocale("FR").getISO3Country());
+    }
+
+    @Test
+    @DefaultLocale(country = "US", language = "en")
+    void testToLocaleGetIso3LanguageKown() {
+        assertEquals("United States", LocaleUtils.toLocale("US").getDisplayCountry());
+        assertEquals("United Kingdom", LocaleUtils.toLocale("GB").getDisplayCountry());
+        assertEquals("Pakistan", LocaleUtils.toLocale("PK").getDisplayCountry());
+        assertEquals("India", LocaleUtils.toLocale("IN").getDisplayCountry());
+        assertEquals("France", LocaleUtils.toLocale("FR").getDisplayCountry());
+    }
+
+    @ParameterizedTest
+    @MethodSource("java.util.Locale#getISOCountries")
+    void testToLocaleGetIso3LanguageKown(final String country) {
+        assertEquals(LocaleUtils.ofCountry(country).getDisplayCountry(), LocaleUtils.toLocale(country).getDisplayCountry());
+    }
+
     /**
      * Test toLocale(Locale) method.
      */
     @ParameterizedTest
     @MethodSource("java.util.Locale#getAvailableLocales")
-    public void testToLocales(final Locale actualLocale) {
+    void testToLocales(final Locale actualLocale) {
         assertEquals(actualLocale, LocaleUtils.toLocale(actualLocale));
     }
 }
